@@ -1,5 +1,5 @@
 // ColourReconnection.h is a part of the PYTHIA event generator.
-// Copyright (C) 2022 Torbjorn Sjostrand.
+// Copyright (C) 2023 Torbjorn Sjostrand.
 // PYTHIA is licenced under the GNU GPL v2 or later, see COPYING for details.
 // Please respect the MCnet Guidelines, see GUIDELINES for details.
 
@@ -59,9 +59,14 @@ public:
 
   // Printing function, mainly intended for debugging.
   void list();
+  long index{0};
 
 };
 
+// Comparison operator by index for two dipole pointers.
+inline bool operator<(const ColourDipolePtr& d1, const ColourDipolePtr& d2) {
+  return ( d1 && d2 ? d1->index < d2->index : !d1 );}
+
 //==========================================================================
 
 // Junction class. In addition to the normal junction class, also contains a
@@ -193,6 +198,26 @@ private:
 
   // List of current dipoles.
   vector<ColourDipolePtr> dipoles, usedDipoles;
+
+  // Last used dipole index, used for sorting.
+  int dipoleIndex{0};
+
+  // Add a dipole and increment index.
+  void addDipole(int colIn = 0, int iColIn = 0, int iAcolIn = 0,
+    int colReconnectionIn = 0, bool isJunIn = false, bool isAntiJunIn = false,
+    bool isActiveIn = true, bool isRealIn = false) {
+    dipoles.push_back(make_shared<ColourDipole>(colIn, iColIn, iAcolIn,
+        colReconnectionIn, isJunIn, isAntiJunIn, isActiveIn, isRealIn));
+    dipoles.back()->index = ++dipoleIndex;
+  }
+
+  // Add a dipole and increment index.
+  void addDipole(const ColourDipole& dipole) {
+    dipoles.push_back(make_shared<ColourDipole>(dipole));
+    dipoles.back()->index = ++dipoleIndex;
+  }
+
+  // Lists of particles, junctions and trials.
   vector<ColourJunction> junctions;
   vector<ColourParticle> particles;
   vector<TrialReconnection> junTrials, dipTrials;
@@ -300,13 +325,13 @@ private:
   void storeUsedDips(TrialReconnection& trial);
 
   // Change colour structure to describe the reconnection in juncTrial.
-  void doJunctionTrial(Event& event, TrialReconnection& juncTrial);
+  bool doJunctionTrial(Event& event, TrialReconnection& juncTrial);
 
   // Change colour structure to describe the reconnection in juncTrial.
   void doDipoleTrial(TrialReconnection& trial);
 
   // Change colour structure if it is three dipoles forming a junction system.
-  void doTripleJunctionTrial(Event& event, TrialReconnection& juncTrial);
+  bool doTripleJunctionTrial(Event& event, TrialReconnection& juncTrial);
 
   // Calculate the difference between the old and new lambda.
   double getLambdaDiff(ColourDipolePtr dip1, ColourDipolePtr dip2,
