@@ -1,5 +1,5 @@
 // Pythia.h is a part of the PYTHIA event generator.
-// Copyright (C) 2022 Torbjorn Sjostrand.
+// Copyright (C) 2023 Torbjorn Sjostrand.
 // PYTHIA is licenced under the GNU GPL v2 or later, see COPYING for details.
 // Please respect the MCnet Guidelines, see GUIDELINES for details.
 
@@ -10,13 +10,14 @@
 #define Pythia8_Pythia_H
 
 // Version number defined for use in macros and for consistency checks.
-#define PYTHIA_VERSION 8.307
-#define PYTHIA_VERSION_INTEGER 8307
+#define PYTHIA_VERSION 8.310
+#define PYTHIA_VERSION_INTEGER 8310
 
 // Header files for the Pythia class and for what else the user may need.
 #include "Pythia8/Analysis.h"
 #include "Pythia8/Basics.h"
 #include "Pythia8/BeamParticle.h"
+#include "Pythia8/BeamSetup.h"
 #include "Pythia8/Event.h"
 #include "Pythia8/FragmentationFlavZpT.h"
 #include "Pythia8/HadronLevel.h"
@@ -24,6 +25,7 @@
 #include "Pythia8/Info.h"
 #include "Pythia8/JunctionSplitting.h"
 #include "Pythia8/LesHouches.h"
+#include "Pythia8/Logger.h"
 #include "Pythia8/SigmaLowEnergy.h"
 #include "Pythia8/Merging.h"
 #include "Pythia8/MergingHooks.h"
@@ -93,7 +95,7 @@ public:
   bool checkVersion();
 
   // Read in one update for a setting or particle data from a single line.
-  bool readString(string, bool warn = true);
+  bool readString(string, bool warn = true, int subrun = SUBRUNDEFAULT);
 
   // Read in updates for settings or particle data from user-defined file.
   bool readFile(string fileName, bool warn = true,
@@ -113,27 +115,32 @@ public:
     PDFPtr pdfHardGamAPtrIn = nullptr, PDFPtr pdfHardGamBPtrIn = nullptr,
     PDFPtr pdfUnresAPtrIn = nullptr, PDFPtr pdfUnresBPtrIn = nullptr,
     PDFPtr pdfUnresGamAPtrIn = nullptr, PDFPtr pdfUnresGamBPtrIn = nullptr,
-    PDFPtr pdfVMDAPtrIn = nullptr, PDFPtr pdfVMDBPtrIn = nullptr);
-  bool setPDFAPtr( PDFPtr pdfAPtrIn );
-  bool setPDFBPtr( PDFPtr pdfBPtrIn );
+    PDFPtr pdfVMDAPtrIn = nullptr, PDFPtr pdfVMDBPtrIn = nullptr) {
+      return beamSetup.setPDFPtr( pdfAPtrIn, pdfBPtrIn, pdfHardAPtrIn,
+      pdfHardBPtrIn, pdfPomAPtrIn, pdfPomBPtrIn, pdfGamAPtrIn, pdfGamBPtrIn,
+      pdfHardGamAPtrIn, pdfHardGamBPtrIn, pdfUnresAPtrIn, pdfUnresBPtrIn,
+      pdfUnresGamAPtrIn, pdfUnresGamBPtrIn, pdfVMDAPtrIn, pdfVMDBPtrIn); }
+  bool setPDFAPtr( PDFPtr pdfAPtrIn ) {
+    return beamSetup.setPDFAPtr( pdfAPtrIn); }
+  bool setPDFBPtr( PDFPtr pdfBPtrIn ) {
+    return beamSetup.setPDFBPtr( pdfBPtrIn); }
 
   // Set photon fluxes externally. Used with option "PDF:lepton2gammaSet = 2".
   bool setPhotonFluxPtr( PDFPtr photonFluxAIn, PDFPtr photonFluxBIn) {
-    if ( photonFluxAIn ) pdfGamFluxAPtr = photonFluxAIn;
-    if ( photonFluxBIn ) pdfGamFluxBPtr = photonFluxBIn;
-    return true;}
+    return beamSetup.setPhotonFluxPtr( photonFluxAIn, photonFluxBIn); }
 
   // Possibility to pass in pointer to external LHA-interfaced generator.
   bool setLHAupPtr( LHAupPtr lhaUpPtrIn) {lhaUpPtr = lhaUpPtrIn;
-    useNewLHA = false; return true;}
+    useNewLHA = false; return beamSetup.setLHAupPtr( lhaUpPtrIn);}
 
   // Possibility to pass in pointer for external handling of some decays.
   bool setDecayPtr( DecayHandlerPtr decayHandlePtrIn,
-    vector<int> handledParticlesIn) {decayHandlePtr = decayHandlePtrIn;
-    handledParticles = handledParticlesIn; return true;}
+    vector<int> handledParticlesIn = {}) {decayHandlePtr = decayHandlePtrIn;
+    handledParticles = handledParticlesIn.size() == 0 ?
+      decayHandlePtrIn->handledParticles() : handledParticlesIn; return true;}
 
   // Possibility to pass in pointer for external random number generation.
-  bool setRndmEnginePtr( RndmEngine* rndmEnginePtrIn)
+  bool setRndmEnginePtr( RndmEnginePtr rndmEnginePtrIn)
     { return rndm.rndmEnginePtr( rndmEnginePtrIn);}
 
   // Possibility to pass in pointer for user hooks.
@@ -160,16 +167,29 @@ public:
 
   // Possibility to pass in pointer for beam shape.
   bool setBeamShapePtr( BeamShapePtr beamShapePtrIn)
-    { beamShapePtr = beamShapePtrIn; return true;}
+    { return beamSetup.setBeamShapePtr(beamShapePtrIn);}
 
-  // Possibility to pass in pointer(s) for external cross section,
-  // with option to include external phase-space generator(s).
-  bool setSigmaPtr( SigmaProcess* sigmaPtrIn, PhaseSpace* phaseSpacePtrIn = 0)
-    { sigmaPtrs.push_back( sigmaPtrIn);
+  // Possibility to pass in pointer for external cross section,
+  // with option to include external phase-space generator.
+  bool setSigmaPtr( SigmaProcessPtr sigmaPtrIn,
+    PhaseSpacePtr phaseSpacePtrIn = nullptr)
+    { sigmaPtrs.resize(0), phaseSpacePtrs.resize(0);
+      sigmaPtrs.push_back(sigmaPtrIn);
       phaseSpacePtrs.push_back(phaseSpacePtrIn); return true;}
 
-  // Possibility to pass in pointer(s) for external resonance.
-  bool setResonancePtr( ResonanceWidths* resonancePtrIn)
+  // Possibility to add further pointers to allow for multiple cross sections.
+  bool addSigmaPtr( SigmaProcessPtr sigmaPtrIn,
+    PhaseSpacePtr phaseSpacePtrIn = nullptr)
+    { sigmaPtrs.push_back(sigmaPtrIn);
+      phaseSpacePtrs.push_back(phaseSpacePtrIn); return true;}
+
+  // Possibility to pass in pointer for external resonance.
+  bool setResonancePtr( ResonanceWidthsPtr resonancePtrIn)
+    { resonancePtrs.resize(0);
+      resonancePtrs.push_back( resonancePtrIn); return true;}
+
+  // Possibility to add further pointers to allow for multiple resonances.
+  bool addResonancePtr( ResonanceWidthsPtr resonancePtrIn)
     { resonancePtrs.push_back( resonancePtrIn); return true;}
 
   // Possibility to pass in pointer for external showers.
@@ -190,7 +210,7 @@ public:
   HeavyIonsPtr getHeavyIonsPtr() { return heavyIonsPtr;}
 
   // Possibility to access the pointer to the BeamShape object.
-  BeamShapePtr getBeamShapePtr() { return beamShapePtr; }
+  BeamShapePtr getBeamShapePtr() { return beamSetup.getBeamShapePtr(); }
 
   // Possibility to get the pointer to the parton-shower model.
   ShowerModelPtr getShowerModelPtr() { return showerModelPtr; }
@@ -207,7 +227,13 @@ public:
   bool next(int procTypeIn);
 
   // Switch to new beam particle identities; for similar hadrons only.
-  bool setBeamIDs( int idAin, int idBin = 0);
+  bool setBeamIDs( int idAin, int idBin = 0) {
+    if (!isInit) { logger.ERROR_MSG("Pythia is not properly initialized");
+    return false; }
+    if (!beamSetup.setBeamIDs( idAin, idBin)) return false;
+    if (beamSetup.hasSwitchedIDs) { processLevel.updateBeamIDs();
+      partonLevel.setBeamID(beamSetup.iPDFAsave); }
+    return true;}
 
   // Switch beam kinematics.
   bool setKinematics(double eCMIn);
@@ -218,7 +244,9 @@ public:
 
   // Generate only a single timelike shower as in a decay.
   int forceTimeShower( int iBeg, int iEnd, double pTmax, int nBranchMax = 0)
-    {  partonSystems.clear(); infoPrivate.setScalup( 0, pTmax);
+    { if (!isInit) {
+      logger.ERROR_MSG("Pythia is not properly initialized"); return 0; }
+      partonSystems.clear(); infoPrivate.setScalup( 0, pTmax);
     return timesDecPtr->shower( iBeg, iEnd, event, pTmax, nBranchMax); }
 
   // Generate only the hadronization/decay stage.
@@ -233,31 +261,46 @@ public:
 
   // Do a low-energy collision between two hadrons in the event record.
   bool doLowEnergyProcess(int i1, int i2, int procTypeIn) {
+    if (!isInit) {
+      logger.ERROR_MSG("Pythia is not properly initialized"); return false; }
     return hadronLevel.doLowEnergyProcess( i1, i2, procTypeIn, event); }
 
   // Get total cross section for two hadrons in the event record or standalone.
-  double getSigmaTotal() { return getSigmaTotal(idA, idB, eCM, 0); }
+  double getSigmaTotal() { return getSigmaTotal(beamSetup.idA, beamSetup.idB,
+    beamSetup.eCM, 0); }
   double getSigmaTotal(double eCM12, int mixLoHi = 0) {
-    return getSigmaTotal(idA, idB, eCM12, mixLoHi); }
+    return getSigmaTotal(beamSetup.idA, beamSetup.idB, eCM12, mixLoHi); }
   double getSigmaTotal(int id1, int id2, double eCM12, int mixLoHi = 0) {
     return getSigmaTotal(id1, id2, eCM12, particleData.m0(id1),
       particleData.m0(id2), mixLoHi); }
   double getSigmaTotal(int id1, int id2, double eCM12, double m1, double m2,
     int mixLoHi = 0) {
+    if (!isInit) {
+      logger.ERROR_MSG("Pythia is not properly initialized"); return 0.; }
     return sigmaCmb.sigmaTotal(id1, id2, eCM12, m1, m2, mixLoHi); }
 
   // Get partial (elastic, diffractive, nondiffractive, ...) cross sections
   // for two hadrons in the event record or standalone.
   double getSigmaPartial(int procTypeIn) {
-    return getSigmaPartial(idA, idB, eCM, procTypeIn, 0); }
+    return getSigmaPartial(beamSetup.idA, beamSetup.idB, beamSetup.eCM,
+    procTypeIn, 0); }
   double getSigmaPartial(double eCM12, int procTypeIn, int mixLoHi = 0) {
-    return getSigmaPartial(idA, idB, eCM12, procTypeIn, mixLoHi); }
+    return getSigmaPartial(beamSetup.idA, beamSetup.idB, eCM12, procTypeIn,
+    mixLoHi); }
   double getSigmaPartial(int id1, int id2, double eCM12, int procTypeIn,
     int mixLoHi = 0) { return getSigmaPartial(id1, id2, eCM12,
       particleData.m0(id1), particleData.m0(id2), procTypeIn, mixLoHi); }
   double getSigmaPartial(int id1, int id2, double eCM12, double m1,
-    double m2, int procTypeIn, int mixLoHi = 0) { return
-    sigmaCmb.sigmaPartial(id1, id2, eCM12, m1, m2, procTypeIn, mixLoHi);}
+    double m2, int procTypeIn, int mixLoHi = 0) {
+    if (!isInit) {
+      logger.ERROR_MSG("Pythia is not properly initialized"); return 0.; }
+    return sigmaCmb.sigmaPartial(id1, id2, eCM12, m1, m2, procTypeIn, mixLoHi);
+  }
+
+  // Return a parton density set among list of possibilities.
+  PDFPtr getPDFPtr(int idIn, int sequence = 1, string beam = "A",
+    bool resolved = true) {
+    return beamSetup.getPDFPtr( idIn, sequence, beam, resolved); }
 
   // List the current Les Houches event.
   void LHAeventList() { if (lhaUpPtr != 0) lhaUpPtr->listEvent();}
@@ -276,10 +319,6 @@ public:
   double parm(string key) {return settings.parm(key);}
   string word(string key) {return settings.word(key);}
 
-  // Auxiliary to set parton densities among list of possibilities.
-  PDFPtr getPDFPtr(int idIn, int sequence = 1, string beam = "A",
-    bool resolved = true);
-
   // The event record for the parton-level central process.
   Event           process = {};
 
@@ -289,6 +328,9 @@ public:
   // Public information and statistic on the generation.
   const Info&     info = infoPrivate;
 
+  // Logger: for diagnostic messages, errors, statistics, etc.
+  Logger          logger = {};
+
   // Settings: databases of flags/modes/parms/words to control run.
   Settings        settings = {};
 
@@ -327,8 +369,8 @@ public:
   HadronWidths    hadronWidths = {};
 
   // The two incoming beams.
-  BeamParticle   beamA = {};
-  BeamParticle   beamB = {};
+  const BeamParticle& beamA = beamSetup.beamA;
+  const BeamParticle& beamB = beamSetup.beamB;
 
 private:
 
@@ -347,6 +389,9 @@ private:
   // Initialise new Pythia object (called by constructors).
   void initPtrs();
 
+  // Initialise user provided plugins.
+  void initPlugins();
+
   // Functions to be called at the beginning and end of a next() call.
   void beginEvent();
   void endEvent(PhysicsBase::Status status);
@@ -367,7 +412,7 @@ private:
   static const double VERSIONNUMBERHEAD, VERSIONNUMBERCODE;
   // Maximum number of tries to produce parton level from given input.
   // Negative integer to denote that no subrun has been set.
-  static const int    NTRY = 10, SUBRUNDEFAULT = -999;
+  static const int    NTRY = 10;
 
   // Initialization data, extracted from database.
   string xmlPath = {};
@@ -376,91 +421,27 @@ private:
          doCentralDiff = {}, doDiffraction = {}, doSoftQCD = {},
          doVMDsideA = {}, doVMDsideB = {}, doHardDiff = {}, doResDec = {},
          doFSRinRes = {}, decayRHadrons = {}, doPartonVertex = {},
-         doVertexPlane = {}, abortIfVeto = {}, checkEvent = {},
-         checkHistory = {}, doNonPert = {},
-         allowIDAswitch = {};
+         abortIfVeto = {}, checkEvent = {}, checkHistory = {}, doNonPert = {};
   int    nErrList = {};
-  vector<int> idAList = { 2212, 211, 311, 221,
-         331, 333, 411, 431, 443, 511, 531, 541, 553, 3212, 3312, 3334,
-         4112, 4312, 4332, 5112, 5312, 5332};
   double epTolErr = {}, epTolWarn = {}, mTolErr = {}, mTolWarn = {};
 
-  // Initialization data related to photon-photon/hadron interactions.
-  int    gammaMode = {};
-  bool   beamA2gamma = {}, beamB2gamma = {};
-  bool   beamAResGamma = {}, beamBResGamma = {};
-  bool   beamAUnresGamma = {}, beamBUnresGamma = {};
-
   // Initialization data, from init(...) call, plus some event-specific.
-  bool   isConstructed = {}, isInit = {}, isUnresolvedA = {},
-         isUnresolvedB = {}, showSaV = {}, showMaD = {}, doReconnect = {},
-         forceHadronLevelCR = {};
-  int    idA = {}, idB = {}, frameType = {}, boostType = {}, nCount = {},
-         nShowLHA = {}, nShowInfo = {}, nShowProc = {}, nShowEvt = {},
-         reconnectMode = {}, iPDFAsave = {};
-  double mA = {}, mB = {}, pxA = {}, pxB = {}, pyA = {}, pyB = {}, pzA = {},
-         pzB = {}, eA = {}, eB = {}, pzAcm = {}, pzBcm = {}, eCM = {},
-         betaZ = {}, gammaZ = {};
-  Vec4   pAinit = {}, pBinit = {}, pAnow = {}, pBnow = {};
-  RotBstMatrix MfromCM = {}, MtoCM = {};
+  bool   isConstructed = {}, isInit = {}, showSaV = {}, showMaD = {},
+         doReconnect = {}, forceHadronLevelCR = {};
+  int    nCount = {}, nShowLHA = {}, nShowInfo = {}, nShowProc = {},
+         nShowEvt = {}, reconnectMode = {};
 
   // information for error checkout.
   int    nErrEvent = {};
   vector<int> iErrId = {}, iErrCol = {}, iErrEpm = {}, iErrNan = {},
          iErrNanVtx = {};
 
-  // Pointers to the parton distributions of the two incoming beams.
-  PDFPtr pdfAPtr = {};
-  PDFPtr pdfBPtr = {};
-
-  // Extra PDF pointers to be used in hard processes only.
-  PDFPtr pdfHardAPtr = {};
-  PDFPtr pdfHardBPtr = {};
-
-  // Extra Pomeron PDF pointers to be used in diffractive processes only.
-  PDFPtr pdfPomAPtr = {};
-  PDFPtr pdfPomBPtr = {};
-
-  // Extra Photon PDF pointers to be used in lepton -> gamma processes.
-  PDFPtr pdfGamAPtr = {};
-  PDFPtr pdfGamBPtr = {};
-
-  // Extra PDF pointers to be used in hard lepton -> gamma processes.
-  PDFPtr pdfHardGamAPtr = {};
-  PDFPtr pdfHardGamBPtr = {};
-
-  // Alternative unresolved PDFs when mixing resolved and unresolved processes.
-  PDFPtr pdfUnresAPtr = {};
-  PDFPtr pdfUnresBPtr = {};
-  PDFPtr pdfUnresGamAPtr = {};
-  PDFPtr pdfUnresGamBPtr = {};
-
-  // PDF pointers to externally provided photon fluxes.
-  PDFPtr pdfGamFluxAPtr = {};
-  PDFPtr pdfGamFluxBPtr = {};
-
-  // Extra VMD PDF pointers to be used in SoftQCD with gammas.
-  PDFPtr pdfVMDAPtr = {};
-  PDFPtr pdfVMDBPtr = {};
-
-  // Array of PDFs to be used when idA can be changed between events.
-  vector<PDFPtr> pdfASavePtrs = {};
-
-  // Alternative Pomeron beam-inside-beam.
-  BeamParticle beamPomA = {};
-  BeamParticle beamPomB = {};
-
-  // Alternative photon beam-inside-beam.
-  BeamParticle beamGamA = {};
-  BeamParticle beamGamB = {};
-
-  // Alternative VMD beam-inside-beam.
-  BeamParticle beamVMDA = {};
-  BeamParticle beamVMDB = {};
+  // Setup of beams: flavours, kinematics and PDFs.
+  BeamSetup beamSetup = {};
 
   // LHAup object for generating external events.
-  bool   doLHA = false;
-  bool   useNewLHA = false;
+  bool     doLHA = false;
+  bool     useNewLHA = false;
   LHAupPtr lhaUpPtr = {};
 
   // Pointer to external decay handler and list of particles it handles.
@@ -474,18 +455,18 @@ private:
 
   // Pointer to BeamShape object for beam momentum and interaction vertex.
   BeamShapePtr beamShapePtr = {};
-  bool         doMomentumSpread = {}, doVertexSpread = {}, doVarEcm = {};
+  bool         doVertexSpread = {};
   double       eMinPert = {}, eWidthPert = {};
 
   // Pointers to external processes derived from the Pythia base classes.
-  vector<SigmaProcess*> sigmaPtrs = {};
+  vector<SigmaProcessPtr> sigmaPtrs = {};
 
   // Pointers to external phase-space generators derived from Pythia
   // base classes.
-  vector<PhaseSpace*> phaseSpacePtrs = {};
+  vector<PhaseSpacePtr> phaseSpacePtrs = {};
 
   // Pointers to external calculation of resonance widths.
-  vector<ResonanceWidths*> resonancePtrs = {};
+  vector<ResonanceWidthsPtr> resonancePtrs = {};
 
   // Pointers to timelike and spacelike showers, including Vincia and Dire.
   TimeShowerPtr  timesDecPtr = {};
@@ -544,24 +525,9 @@ private:
   // Check that combinations of settings are allowed; change if not.
   void checkSettings();
 
-  // Check that beams and beam combination can be handled.
-  bool checkBeams();
-
-  // Calculate kinematics at initialization.
-  bool initKinematics();
-
-  // Set up pointers to PDFs.
-  bool initPDFs();
-
-  // Recalculate kinematics for each event when beam momentum has a spread.
-  void nextKinematics();
-
   // Simplified treatment for low-energy nonperturbative collisions.
   bool nextNonPert(int procTypeIn = 0);
 
-  // Boost from CM frame to lab frame, or inverse. Set production vertex.
-  void boostAndVertex(bool toLab, bool setVertex);
-
   // Perform R-hadron decays.
   bool doRHadronDecays();
 
