@@ -1,5 +1,5 @@
 // PythiaStdlib.h is a part of the PYTHIA event generator.
-// Copyright (C) 2022 Torbjorn Sjostrand.
+// Copyright (C) 2023 Torbjorn Sjostrand.
 // PYTHIA is licenced under the GNU GPL v2 or later, see COPYING for details.
 // Please respect the MCnet Guidelines, see GUIDELINES for details.
 
@@ -11,6 +11,7 @@
 #define Pythia8_PythiaStdlib_H
 
 // Stdlib header files for mathematics.
+#include <cstddef>
 #include <cmath>
 #include <cstdlib>
 #include <algorithm>
@@ -50,6 +51,11 @@
 #define M_PI 3.1415926535897932385
 #endif
 
+// Define the default subrun.
+#ifndef SUBRUNDEFAULT
+#define SUBRUNDEFAULT -999
+#endif
+
 // Set floating point exceptions from the gcc compiler for debug
 // purposes. Use the compilation flag -DGCCFPDEBUG to enable.
 #ifdef GCCFPDEBUG
@@ -70,7 +76,6 @@ static void __attribute__((constructor)) raisefpe() {
 namespace Pythia8 {
 
 // Generic utilities and mathematical functions.
-using std::move;
 using std::swap;
 using std::max;
 using std::min;
@@ -104,6 +109,7 @@ using std::function;
 using std::cin;
 using std::cout;
 using std::cerr;
+using std::streambuf;
 using std::istream;
 using std::ostream;
 using std::fstream;
@@ -135,6 +141,7 @@ using std::queue;
 using std::mutex;
 using std::thread;
 using std::atomic;
+using std::lock_guard;
 
 } // end namespace Pythia8
 
@@ -180,6 +187,10 @@ constexpr double pow8(const double& x) {return x*x*x*x*x*x*x*x;}
 // Avoid problem with negative square root argument (from roundoff).
 inline double sqrtpos(const double& x) {return sqrt( max( 0., x));}
 
+// Explicitly return NaN if negative square root argument, without FPE.
+inline double sqrtnan(const double& x) {
+  return x > 0 ? sqrt(x) : numeric_limits<double>::quiet_NaN(); }
+
 // Restrinct value to lie in specified range.
 inline double clamp(const double& x, const double& xmin, const double& xmax) {
   return (x < xmin) ? xmin : (x > xmax) ? xmax : x; }
@@ -194,6 +205,95 @@ inline void toLowerRep(string& name, bool trim = true) {
 
 //==========================================================================
 
+// Print a method name using the appropriate pre-processor macro.
+
+//  The following method was modified from
+//  boost/current_function.hpp - BOOST_CURRENT_FUNCTION
+//
+//  Copyright (c) 2002 Peter Dimov and Multi Media Ltd.
+//
+//  Distributed under the Boost Software License, Version 1.0.
+//  Boost Software License - Version 1.0 - August 17th, 2003
+//
+//  Permission is hereby granted, free of charge, to any person or
+//  organization obtaining a copy of the software and accompanying
+//  documentation covered by this license (the "Software") to use,
+//  reproduce, display, distribute, execute, and transmit the
+//  Software, and to prepare derivative works of the Software, and to
+//  permit third-parties to whom the Software is furnished to do so,
+//  all subject to the following:
+//
+//  The copyright notices in the Software and this entire statement,
+//  including the above license grant, this restriction and the
+//  following disclaimer, must be included in all copies of the
+//  Software, in whole or in part, and all derivative works of the
+//  Software, unless such copies or derivative works are solely in the
+//  form of machine-executable object code generated by a source
+//  language processor.
+//
+//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+//  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+//  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
+//  NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
+//  ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE FOR ANY DAMAGES OR
+//  OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING
+//  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+//  OTHER DEALINGS IN THE SOFTWARE.
+//
+//  http://www.boost.org/libs/utility/current_function.html
+//
+//  Note that Boost Software License - Version 1.0 is fully compatible
+//  with GPLV2
+//  For more information see https://www.gnu.org/licenses/license-list.en.html
+
+#ifndef __METHOD_NAME__
+
+#ifndef PYTHIA_FUNCTION
+#if ( defined(__GNUC__) || (defined(__MWERKS__) && (__MWERKS__ >= 0x3000)) \
+|| (defined(__ICC) && (__ICC >= 600)) )
+# define PYTHIA_FUNCTION __PRETTY_FUNCTION__
+#elif defined(__DMC__) && (__DMC__ >= 0x810)
+# define PYTHIA_FUNCTION __PRETTY_FUNCTION__
+#elif defined(__FUNCSIG__)
+# define PYTHIA_FUNCTION __FUNCSIG__
+#elif ( (defined(__INTEL_COMPILER) && (__INTEL_COMPILER >= 600)) \
+|| (defined(__IBMCPP__) && (__IBMCPP__ >= 500)) )
+# define PYTHIA_FUNCTION __FUNCTION__
+#elif defined(__BORLANDC__) && (__BORLANDC__ >= 0x550)
+# define PYTHIA_FUNCTION __FUNC__
+#elif defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 199901)
+# define PYTHIA_FUNCTION __func__
+#else
+# define PYTHIA_FUNCTION "unknown"
+#endif
+#endif // end PYTHIA_FUNCTION
+
+inline std::string methodName(const std::string& prettyFunction, bool
+  withNamespace=false) {
+
+  // Find the beginning of the argument list.
+  size_t end = prettyFunction.rfind(')');
+  int bracketCount = 1;
+  while (bracketCount > 0) {
+    --end;
+    if (prettyFunction[end] == ')') ++bracketCount;
+    else if (prettyFunction[end] == '(') --bracketCount;
+  }
+
+  // Find the start of the function name.
+  size_t begin = prettyFunction.rfind(' ', end) + 1;
+  if (!withNamespace)
+    begin = prettyFunction.find("::", begin) + 2;
+
+  // Return the correct substring.
+  return prettyFunction.substr(begin, end - begin);
+}
+
+#define __METHOD_NAME__ methodName(PYTHIA_FUNCTION)
+#endif // __METHOD_NAME__
+
+//==========================================================================
+
 } // end namespace Pythia8
 
 // Define the hash for a pair.
