@@ -1,5 +1,5 @@
 // HepMC3.h is a part of the PYTHIA event generator.
-// Copyright (C) 2022 Torbjorn Sjostrand.
+// Copyright (C) 2023 Torbjorn Sjostrand.
 // PYTHIA is licenced under the GNU GPL v2 or later, see COPYING for details.
 // Please respect the MCnet Guidelines, see GUIDELINES for details.
 //
@@ -17,7 +17,7 @@
 
 #include <vector>
 #include "Pythia8/Pythia.h"
-#include "Pythia8/HIUserHooks.h"
+#include "Pythia8/HIInfo.h"
 #include "HepMC3/GenVertex.h"
 #include "HepMC3/GenParticle.h"
 #include "HepMC3/GenEvent.h"
@@ -165,7 +165,7 @@ public:
       // the HepMC event standard.
       if ( !hepevt_particles[i] ) {
         std::cerr << "hanging particle " << i << std::endl;
-        GenVertexPtr prod_vtx;
+        GenVertexPtr prod_vtx = make_shared<GenVertex>();
         prod_vtx->add_particle_out( hepevt_particles[i] );
         evt->add_vertex(prod_vtx);
       }
