@@ -1,11 +1,11 @@
 // main83.cc is a part of the PYTHIA event generator.
-// Copyright (C) 2022 Torbjorn Sjostrand.
+// Copyright (C) 2023 Torbjorn Sjostrand.
 // PYTHIA is licenced under the GNU GPL v2 or later, see COPYING for details.
 // Please respect the MCnet Guidelines, see GUIDELINES for details.
 
-// Authors: Stefan Prestel <stefan.prestel@thep.lu.se>.
+// Authors: Stefan Prestel
 
-// Keywords: merging; leading order;
+// Keywords: merging; leading order
 
 // It illustrates how to do CKKW-L merging, see the Matrix Element
 // Merging page in the online manual. An example command is
@@ -318,7 +318,9 @@ int main( int argc, char* argv[] ){
     if( ! pythia.next()) continue;
 
     // Get CKKWL weight of current event
-    double weight = pythia.info.mergingWeight();
+    double evtweight = pythia.info.weight();
+    double weight    = pythia.info.mergingWeight();
+    weight      *= evtweight;
 
     // Fill bins with CKKWL weight
     double pTfirst = pTfirstJet(pythia.event,1, 0.4);
