@@ -1,11 +1,11 @@
 // main84.cc is a part of the PYTHIA event generator.
-// Copyright (C) 2022 Torbjorn Sjostrand.
+// Copyright (C) 2023 Torbjorn Sjostrand.
 // PYTHIA is licenced under the GNU GPL v2 or later, see COPYING for details.
 // Please respect the MCnet Guidelines, see GUIDELINES for details.
 
-// Authors: Stefan Prestel <stefan.prestel@thep.lu.se>.
+// Authors: Stefan Prestel
 
-// Keywords: merging; leading order; CKKW-L; hepmc;
+// Keywords: merging; leading order; CKKW-L; hepmc
 
 // It illustrates how to do CKKW-L merging, see the Matrix Element
 // Merging page in the online manual. An example command is
@@ -289,7 +289,9 @@ int main( int argc, char* argv[] ){
         // Generate next event
         if( pythia.next()) {
 
-          double weight = pythia.info.mergingWeight();
+          double evtweight = pythia.info.weight();
+          double weight    = pythia.info.mergingWeight();
+          weight      *= evtweight;
           nAccepEvents++;
 
           // Jet pT's
